@@ -12,23 +12,10 @@ from sklearn.svm import SVC
 
 
 class NmfModel:
-    def __init__(
-            self,
-            mat: Union[np.ndarray, pd.DataFrame],
-            ncomp: int,
-            nblocks: int = 1,
-            name: str = "",
-            use_hals: bool = True,
-            beta_loss: str = "frobenius",
-            leverage: str = "standard",
-            kernel: str = "linear",
-            tol: float = 1e-6,
-            max_iter: int = 150,
-            max_iter_mult: int = 20,
-            regularization: str = None,
-            features_names: Tuple[str] = None,
-            object_names: Tuple[str] = None,
-    ):
+    def __init__(self, mat: Union[np.ndarray, pd.DataFrame], ncomp: int, nblocks: int = 1, name: str = "",
+                 beta_loss: str = "frobenius", leverage: str = "standard", kernel: str = "linear", tol: float = 1e-6,
+                 max_iter: int = 150, max_iter_mult: int = 20, regularization: str = None,
+                 features_names: Tuple[str] = None, object_names: Tuple[str] = None):
 
         if object_names is None:
             object_names = []
@@ -37,7 +24,6 @@ class NmfModel:
         self._fnames = []
         self._objnames = []
         self._ncomp = ncomp
-        self._hals = use_hals
         self._beta_loss = beta_loss
         self._tol = tol
         if isinstance(max_iter, str):
@@ -59,7 +45,7 @@ class NmfModel:
         self._sparsity = 0
         self._n_bootstrap = 0
         self._estimator = None
-        self._predictor = None
+        #  self._predictor = None
         self._error = 0
         self._volume = 0
         self._errors = None
@@ -94,34 +80,20 @@ class NmfModel:
             self._name = name
 
         self._sortedmat = self._filledmat.copy()
-        self._model = NMF(n_components=ncomp, use_hals=use_hals)
+        self._model = NMF(n_components=ncomp)
         self._estimator = None
-        self._predictor = None
+        # self._predictor = None
         self._error = None
         self._volume = None
 
     def _reset(self):
-        if hasattr(self, "_estimator"):
-            delattr(self, "_estimator")
-        if hasattr(self, "_estimator"):
-            delattr(self, "_predictor")
+        self._estimator = None
 
     def clone(self):
-        model = NmfModel(
-            mat=self._mat,
-            ncomp=self._ncomp,
-            name=self._name,
-            use_hals=self._hals,
-            beta_loss=self._beta_loss,
-            leverage=self._leverage,
-            kernel=self._kernel,
-            tol=self._tol,
-            max_iter=self._max_iter,
-            max_iter_mult=self._max_iter_mult,
-            regularization=self._regularization,
-            features_names=self._fnames,
-            object_names=self._objnames,
-        )
+        model = NmfModel(mat=self._mat, ncomp=self._ncomp, name=self._name, beta_loss=self._beta_loss,
+                         leverage=self._leverage, kernel=self._kernel, tol=self._tol, max_iter=self._max_iter,
+                         max_iter_mult=self._max_iter_mult, regularization=self._regularization,
+                         features_names=self._fnames, object_names=self._objnames)
 
         model.set_sparsity(self._sparsity)
         model.set_n_bootstrap(self._n_bootstrap)
@@ -137,46 +109,36 @@ class NmfModel:
         self._n_bootstrap = n_bootstrap
 
     def set_ncomp(self, ncomp):
-        self.__init__(
-            mat=self._mat,
-            ncomp=ncomp,
-            name=self._name,
-            use_hals=self._hals,
-            beta_loss=self._beta_loss,
-            leverage=self._leverage,
-            kernel=self._kernel,
-            tol=self._tol,
-            max_iter=self._max_iter,
-            max_iter_mult=self._max_iter_mult,
-            regularization=self._regularization,
-            features_names=self._fnames,
-            object_names=self._objnames,
-        )
+        self.__init__(mat=self._mat, ncomp=ncomp, name=self._name, beta_loss=self._beta_loss, leverage=self._leverage,
+                      kernel=self._kernel, tol=self._tol, max_iter=self._max_iter, max_iter_mult=self._max_iter_mult,
+                      regularization=self._regularization, features_names=self._fnames, object_names=self._objnames)
         self._reset()
 
     def solve_fixed_h_for_w(self, h: Union[np.ndarray, pd.DataFrame]):
         self._estimator = self._model.fit_transform(
-                X=self._filledmat.astype(float).values,
-                H=h,
-                update_W=True,
-                update_H=False,
+                m=self._filledmat.astype(float).values,
+                w=None,
+                h=h.astype(float).values,
+                update_w=True,
+                update_h=False,
                 n_bootstrap=self._n_bootstrap,
                 sparsity=self._sparsity,
                 regularization=self._regularization,
-            )
-        self._predictor = self._model.predict(self._estimator)
+                )
+        self._model.predict(self._estimator)
 
     def solve_fixed_w_for_h(self, w: Union[np.ndarray, pd.DataFrame]):
         self._estimator = self._model.fit_transform(
-                X=self._filledmat.astype(float).values,
-                W=w,
-                update_W=False,
-                update_H=True,
+                m=self._filledmat.astype(float).values,
+                h=None,
+                w=w.astype(float).values,
+                update_w=False,
+                update_h=True,
                 n_bootstrap=self._n_bootstrap,
                 sparsity=self._sparsity,
                 regularization=self._regularization,
             )
-        self._predictor = self._model.predict(self._estimator)
+        self._model.predict(self._estimator)
 
     def update_h(self, h: Union[np.ndarray, pd.DataFrame]):
         self.solve_fixed_h_for_w(h)
@@ -217,21 +179,19 @@ class NmfModel:
                 sparsity=self._sparsity,
                 regularization=self._regularization,
             )
-            self._predictor = self._model.predict(self._estimator)
-            prec = self.get_precision()
+            self._model.predict(self._estimator)
+            prec = self.get_precision() if self._estimator is not None else 1
             if verbose:
                 print("Error: {0:.0f}%".format(prec))
             self._errors[max(0, itrial - 1)] = prec
 
         self._error = self._errors[-1]
-        self._volume = self._estimator["volume"]
+        self._volume = self._estimator.volume
 
     def is_ok(self):
-        return False if not hasattr(self, "_estimator") or not hasattr(self, "_predictor") \
-                        or self._predictor is None \
-                        or "H" not in list(self._predictor.keys()) \
-                        or self._predictor["H"].shape[1] != self._ncomp \
-                        else True
+        return False if not hasattr(self, "_estimator") \
+                        or self._estimator is None \
+                        or self._estimator.h is None else True
 
     @property
     def n_features(self):
@@ -259,7 +219,7 @@ class NmfModel:
         h = pd.DataFrame(
             index=self._fnames,
             columns=list(["V" + str(x) for x in range(1, self._ncomp + 1)]),
-            data=self._predictor["H"],
+            data=self._estimator.h if self._estimator is not None else np.nan,
         )
 
         return h
@@ -270,7 +230,7 @@ class NmfModel:
         w = pd.DataFrame(
             index=self._objnames,
             columns=list(["V" + str(x) for x in range(1, self._ncomp + 1)]),
-            data=self._predictor["W"],
+            data=self._estimator.w if self._estimator is not None else np.nan,
         )
 
         return w
@@ -325,7 +285,7 @@ class NmfModel:
         if not self.is_ok():
             self.test_model()
         nbins = min(self._ncomp, 5)
-        df_clust = pd.DataFrame(self._estimator["WB"] if which.lower() == "w" else self._estimator["HB"])
+        df_clust = pd.DataFrame(self._estimator.wb if which.lower() == "w" else self._estimator.hb)
         entropies = [utl.normalized_entropy(df_clust.loc[idx, :], nbins) for idx in df_clust.index]
         return float(np.nanmean(entropies))
 
@@ -378,8 +338,8 @@ class NmfModel:
     def get_volume(self) -> float:
         if not self.is_ok():
             self.test_model()
-        if "volume" in self._estimator:
-            return self._estimator["volume"]
+        if self._estimator is not None:
+            return self._estimator.volume
         else:
             return np.nan
 
@@ -442,7 +402,7 @@ class NmfModel:
         if not self.is_ok():
             self.test_model()
         for ic, col in enumerate(feat_clust.columns):
-            feat_clust.loc[:, col] = self._predictor["HC"][ic]
+            feat_clust.loc[:, col] = self._estimator.hc[ic]
         mat_w_clust = pd.concat([self._sortedmat, feat_clust])
         mat_w_clust = mat_w_clust.sort_values(by=1000000, axis=1)
         self._sortedmat = mat_w_clust.drop(1000000, axis=0)
@@ -457,7 +417,7 @@ class NmfModel:
         """
         if not self.is_ok():
             self.test_model()
-        return pd.DataFrame(index=self._objnames, columns=["Observation Cluster"], data=self._predictor["WC"])
+        return pd.DataFrame(index=self._objnames, columns=["Observation Cluster"], data=self._estimator.wc)
 
     def get_obs_kmeans_clusters(self, ngroups=0):
         """
@@ -506,7 +466,7 @@ class NmfModel:
     def get_features_leverage_clusters(self):
         if not self.is_ok():
             self.test_model()
-        return pd.DataFrame(index=self._fnames, columns=["Feature Cluster"], data=self._predictor["HC"])
+        return pd.DataFrame(index=self._fnames, columns=["Feature Cluster"], data=self._estimator.hc)
 
     def get_features_kmeans_clusters(self, ngroups=0):
         """
@@ -529,7 +489,7 @@ class NmfModel:
         if not self.is_ok():
             self.test_model()
         for ir, row in enumerate(obj_clust.index):
-            obj_clust.loc[row, :] = self._predictor["WC"][ir]
+            obj_clust.loc[row, :] = self._estimator.wc[ir]
         # nf = self._nfeat
         mat_w_clust = pd.concat([self._sortedmat, obj_clust], axis=1)
         mat_w_clust = mat_w_clust.sort_values(by="OBJ_CLUSTER", axis=0)
