@@ -48,7 +48,7 @@ if __name__ == "__main__":
     nb_clusters = 10
     h_size = 50
     h_avg = int(h_size / nb_clusters)
-    w_size = 40
+    w_size = 400
     w_avg = int(w_size / nb_clusters)
     icorrmin = 0.80
     icorrmax = 0.99
@@ -56,6 +56,8 @@ if __name__ == "__main__":
     xcorrmax = 0.30
 
     mat, _, _, _ = rng.generate_nmf_reconstruction(n_comp=nb_clusters, n_feat=h_size, n_obs=w_size,
-                                                   avg_h_clust=h_avg, avg_w_clust=w_avg)
-    my_k_estimator.estimate_ncomp(mat=mat, ncmin=2, ncmax=8)
+                                                   avg_w_clust=w_avg, avg_h_clust=h_avg,
+                                                   h_icorr_min=icorrmin, h_xcorr_max=xcorrmax,
+                                                   w_icorr_min=icorrmin, w_xcorr_max=xcorrmax)
+    preds = my_k_estimator.estimate_ncomp(mat=mat, inference_model=mymlp)
     pass
